from __future__ import annotations

from tcsoa.gen.BusinessObjects import ItemRevision, POM_object, IncrementalChangeElement, MECfgLine
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ICEAttributes(TcBaseObj):
    """
    The attibutes from a single IncrementaChangeElement structure
    
    :var attributeName: Attribute name.
    :var attributeValue: Attribute value.
    :var absOccRootLine: String representing the absolute occurrence root line (al_absocc_rootline_string)
    :var bomViewRevision: Related BOMView Revision from the absolute occurrence data.
    """
    attributeName: str = ''
    attributeValue: str = ''
    absOccRootLine: str = ''
    bomViewRevision: POM_object = None


@dataclass
class AttachmentChangeResponse(TcBaseObj):
    """
    A list of output for each passed in CfgAttachmentLine and the 'ServiceData'.
    
    
    :var response: A list of output for each passed in CfgAttachmentLine
    :var serviceData: Service data.
    """
    response: List[AttachmentInfo] = ()
    serviceData: ServiceData = None


@dataclass
class AttachmentICEInfo(TcBaseObj):
    """
    Contains the attachment line, the absolute occurrence root line,  and the IncrementalChangeElement objects
    associated with it.
    
    :var line: The attachment line
    :var absOccRootLine: The absolute occurrence root line.
    :var childEditIces: A list of edit IncrementalChangeElement objects which includes the type of
    IncrementalChangeElement, incremental change revision, the IncrementalChangeElement and how it is configured.
    :var relationIces: A list of relation IncrementalChangeElement objects which includes the type of
    IncrementalChangeElement, incremental change revision, the IncrementalChangeElement and how it is configured.
    """
    line: MECfgLine = None
    absOccRootLine: str = ''
    childEditIces: List[BaseICEInfo] = ()
    relationIces: List[BaseICEInfo] = ()


@dataclass
class AttachmentInfo(TcBaseObj):
    """
    A list of 'AttachmentICEInfo' objects each of which contain the attachment line and the IncrementalChangeElement
    objects associated with it.
    
    :var listOfAttachments: A list of 'AttachmentICEInfo' objects each of which contain the attachment line and the
    IncrementalChangeElement objects associated with it.
    """
    listOfAttachments: List[AttachmentICEInfo] = ()


@dataclass
class AttributeChangeResponse(TcBaseObj):
    """
    The response from the 'getAttributeChanges' SOA function
    
    :var response: A list of attribute outputs for the passed in BOMLine objects.
    :var serviceData: The SOA Service data.  Any error will be populated here.
    """
    response: List[AttributeInfoOutput] = ()
    serviceData: ServiceData = None


@dataclass
class AttributeICEInfo(TcBaseObj):
    """
    Structure passed back from the 'getAttributeChanges' SOA call
    
    :var baseICEInfo: Basic IncrementalChangeElement object information for all IncrementalChangeElement object types.
    :var attributes: Attribute information for this IncrementalChangeElement object.
    """
    baseICEInfo: BaseICEInfo = None
    attributes: ICEAttributes = None


@dataclass
class AttributeInfoOutput(TcBaseObj):
    """
    The output from the 'getAttributeChanges' SOA function.
    
    :var attributeInfo: The acutal attribute information
    """
    attributeInfo: List[AttributeICEInfo] = ()


@dataclass
class ParentAndChildComponentsResponse(TcBaseObj):
    """
    Response from the 'getParentAndChildComponents' SOA API call
    
    :var parentAndChildInfoResponse: The output response for the 'getParentAndChildComponents' call.
    :var serviceData: The service data.
    """
    parentAndChildInfoResponse: ParentAndChildInfos = None
    serviceData: ServiceData = None


@dataclass
class ParentAndChildInfos(TcBaseObj):
    """
    The output of the 'getParentAndChildComponents' SOA API call
    
    :var parentChildInfos: A list of 'ParentInfo' structures describing the parent and child components of the passed
    in IncrementalChangeElement objects.
    """
    parentChildInfos: List[ParentChildInfo] = ()


@dataclass
class ParentChildInfo(TcBaseObj):
    """
    Parent and child information for the given IncrementalChangeElement.
    
    :var context: The context
    :var parent: The parent component object.
    :var child: The child component object.
    :var ice: The IncrementalChangeElement object for the given parent and child components.
    """
    context: str = ''
    parent: POM_object = None
    child: POM_object = None
    ice: IncrementalChangeElement = None


@dataclass
class PredecessorChangeInfo(TcBaseObj):
    """
    The output that comes from the 'getPredecessorChanges' SOA function
    
    :var baseICEInfo: Basic information for IncrementalChangeElement objects.
    :var listOfPredecessors: List of predecessor information.
    """
    baseICEInfo: List[BaseICEInfo] = ()
    listOfPredecessors: List[PredecessorICEInfo] = ()


@dataclass
class PredecessorChangeResponse(TcBaseObj):
    """
    The response to the 'getPredecessorChanges' SOA function
    
    :var serviceData: The service data
    :var response: List of predecessor information with the IncrementalChangeElement objects for each.
    """
    serviceData: ServiceData = None
    response: List[PredecessorChangeInfo] = ()


@dataclass
class PredecessorICEInfo(TcBaseObj):
    """
    The predecessors to the given IncrementalChangeElement object.
    
    :var predecessorName: Name of the predecessor.
    :var sequenceNumber: The sequence numbert for this predecessor.
    """
    predecessorName: str = ''
    sequenceNumber: str = ''


@dataclass
class BaseICEInfo(TcBaseObj):
    """
    Basic ICE information
    
    :var typeOfIce: 'Add', 'Delete'  
    'Add' means that a BOMLine has been added to the structure in the context of the incremental change.  'Delete'
    means that a BOMLine has been removed from the structure in the context of the incremental change.
    
    :var howConfigured: If the value of "ic_not_configure" is an empty string, it means that the incremental change is
    not configured therefore it will not show in the structure.
    :var icRev: A single incremental change item revision.
    :var ice: IncrementalChangeElement object
    """
    typeOfIce: int = 0
    howConfigured: str = ''
    icRev: ItemRevision = None
    ice: IncrementalChangeElement = None


@dataclass
class StructureChangeResponse(TcBaseObj):
    """
    A list of outputs per IncrementalChangeElement and the 'serviceDate' structure that consists of the incremental
    change revision and the IncrementalChangeElement object.
    
    
    :var response: A list of outputs per IncrementalChangeElement
    :var serviceData: The SOA service data structure
    It consists of the incremental change revision and the IncrementalChangeElement object.
    """
    response: List[StructureChangesOutput] = ()
    serviceData: ServiceData = None


@dataclass
class StructureChangesOutput(TcBaseObj):
    """
    A list of 'BaseICEInfo' structures that contains any structure changes.
    
    :var baseInfo: A list of 'BaseICEInfo' structures that contains any structure changes.
    """
    baseInfo: List[BaseICEInfo] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ImportScope(TcBaseObj):
    """
    The detail input for importing manufacturing features.
    
    :var fileName: The full path of the PLMXML file
    :var container: The container under which manufacturing features are to be imported.
    :var importMode: The mode of import. The possible values of the import mode are as follows.     
    keepExistingFeatures - The existing discrete manufacturing features (MFGs)under the container should not be
    deleted.      refreshWholeContainer - The existing discrete manufacturing features under the container may be
    deleted.
    """
    fileName: str = ''
    container: BusinessObject = None
    importMode: str = ''


@dataclass
class AdvancedImportInput(TcBaseObj):
    """
    Advanced Input for importing the manufacturing features.
    
    :var context: A BOMLine object in the product structure. Under this object the connected parts of imported
    manufacturing features are searched.
    :var content: List of detail inputs. The detail input elaborates about the container, the import mode and the input
    PLMXML file.
    """
    context: BusinessObject = None
    content: List[ImportScope] = ()


@dataclass
class AdvancedImportResponse(TcBaseObj):
    """
    The response of PLMXML import of the manufacturing features.
    
    :var serviceData: The service data.The error conditions and the corresponding error codes are as listed below.     
    200373 Import is tried to be done from an incomplete PLMXML file.      200374 The preference
    MEImportMFGsManufacturingFeatureIdAttributeName does not have any value.      200376 The preference
    MEImportMFGsManufacturingFeatureIdAttributeName has incorrect value.      200375 The preference
    MEImportMFGsManufacturingFeatureIdAttributeName does not have any value.      200377 The preference
    MEImportMFGsManufacturingFeatureIdAttributeName has incorrect value.      200xxx Import has failed for a container.
    :var logFileName: The name of the generated log file.
    :var logFileTicket: The fms ticket for the transient file that captures the log of import.
    """
    serviceData: ServiceData = None
    logFileName: str = ''
    logFileTicket: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Awb0Element, Awb0ProductContextInfo, BusinessObject, RuntimeBusinessObject, WorkspaceObject, RevisionRule
from tcsoa.gen.Internal.ActiveWorkspaceBom._2017_12.OccurrenceManagement import OccurrenceInfo6, OccurrenceCursor, UserContextInfo, FocusOccurrenceInput
from tcsoa.gen.Internal.ActiveWorkspaceBom._2017_06.OccurrenceManagement import InputContext5, PagedOccurrencesInfo6
from typing import Dict, List
from tcsoa.gen.Internal.ActiveWorkspaceBom._2015_07.OccurrenceManagement import OccurrenceFilterInput, OccurrenceFilterOutput
from tcsoa.gen.Internal.ActiveWorkspaceBom._2018_05.OccurrenceManagement import OccurrenceSortInput, ParentChildrenInfo
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class AddObjectData(TcBaseObj):
    """
    The structure contains object to be added as child of input parentElement or productContext.
    
    :var parentElement: The Awb0Element object under which objectsToBeAdded is added. Only parentElement or
    productContext may be set, not both.
    :var objectsToBeAdded: Business object to be added into product. This will be added under parentElement or
    productContext. It can be Item, ItemRevision, GenericDesignElement or Awb0Element.
    :var siblingElement: The newly added Awb0Element object should be added after this Awb0Element object.This is an
    optional parameter which is used for Requirement structures.
    :var inputCtxt: The input context has the information about filter, configuration, and page size.
    :var sortCriteria: Criteria to sort the occurrences based on internal property name and sorting order e.g. "ASC"
    and "DESC".
    :var addObjectIntent: Indicates user gesture. Possible value is 'DragAndDropIntent'
    :var fetchPagedOccurrences: If true then return the entire pageful along with created Awb0Element.
    :var requestPref: Map (string, string) of preference names and value pairs. Allowed preference names are:
    includeInterfaces , includeUnconfigured and . Allowed values are: True/False. Keys and values are case sensitive.
    """
    parentElement: Awb0Element = None
    objectsToBeAdded: List[BusinessObject] = ()
    siblingElement: Awb0Element = None
    inputCtxt: InputContext5 = None
    sortCriteria: OccurrenceSortInput = None
    addObjectIntent: str = ''
    fetchPagedOccurrences: bool = False
    requestPref: RequestPreference8 = None


@dataclass
class OccurrencesData(TcBaseObj):
    """
    The OccurrencesData supplies the product and the configuration rules to configure the product and get a page of
    occurrences along with effective column configuration. Optionally, filters and sorting criteria may be provided to
    filter or sort the resulting occurrences.
    
    :var product: The top-level product. It may be an Item, ItemRevision representing a product or a Mdl0odelElement
    such as a Cpd0CollaborativeDesign.
    :var parentElement: Parent occurrence for which child occurrences is to be retrieved.
    :var focusOccurrenceInput: Contains input information required to set the focusChildOccurrence.
    :var config: A list of configuration rules to apply to the product.
    :var cursor: Cursor that represents point to which the results have been fetched. This field is used for subsequent
    calls.
    :var filter: A recipe to search and filter occurrences and sort the results.
    :var sortCriteria: Criteria to sort the occurrences based on intetnal property name and sorting order e.g. "ASC"
    and "DESC".
    :var requestPref: Map (string, string) of preference names and value pairs. Allowed preference names are:
    includeInterfaces , includeUnconfigured and . Allowed values are: True/False. Keys and values are case sensitive.
    """
    product: BusinessObject = None
    parentElement: str = ''
    focusOccurrenceInput: FocusOccurrenceInput = None
    config: OccurrenceConfigInput = None
    cursor: OccurrenceCursor = None
    filter: OccurrenceFilterInput = None
    sortCriteria: OccurrenceSortInput = None
    requestPref: RequestPreference8 = None


@dataclass
class OccurrencesResp(TcBaseObj):
    """
    Contains the resulting occurrences found for product from which retrieval was requested. This data structure also
    returns a cursor to be sent back to the subsequent call to getOccurrences in order to rerieve more pages of results.
    
    :var rootProductContext: The Awb0ProductContextInfo for the root occurrence.
    :var parentProductContext: The Awb0ProductContextInfo for the parent occurrence.
    :var sourceContextToInfoMap: Map(WorkspaceObject, SourceContextInfo) containing pair of Source Context Object to
    its corresponding configuration information.
    :var parentChildrenInfos: List of parent and its children information. This will be populated only when
    loadTreeHierarchyThreshold requestPref is not empty.
    :var requestPref: Map (string, string) of preference names and value pairs. Allowed preference names are:
    includeInterfaces, includeUnconfigured and  allowed values are: True/False. Keys and values are case sensitive.
    :var serviceData: Contains the list of all BusinessObjects that make up the output, as well as any errors that
    might have occurred as part of the service invocation.
    :var parentOccurrence: The parent occurrence for list of childOccurrences.
    :var focusProductContext: The Awb0ProductContextInfo for the focus occurrence.
    :var focusChildOccurrence: The focus child occurrence for which sibling occurrences are desired.
    :var occurrences: List of resulting occurrences that exist in the configured structure. The configured occurrence
    object is of type Awb0Element
    :var cursor: Cursor that represents point to which the results have been fetched. This field is used for subsequent
    calls.
    :var userWorkingContextInfo: User's working context information for opened content.
    :var filter: Updated search filter information.
    :var elementToPCIMap: Map containing pair of Root Occurrence (Awb0Element) to its corresponding
    Awb0ProductContextInfo object in Product Set container
    """
    rootProductContext: Awb0ProductContextInfo = None
    parentProductContext: Awb0ProductContextInfo = None
    sourceContextToInfoMap: SourceContextToInfoMap = None
    parentChildrenInfos: List[ParentChildrenInfo] = ()
    requestPref: RequestPreference8 = None
    serviceData: ServiceData = None
    parentOccurrence: OccurrenceInfo6 = None
    focusProductContext: Awb0ProductContextInfo = None
    focusChildOccurrence: OccurrenceInfo6 = None
    occurrences: List[OccurrenceInfo6] = ()
    cursor: OccurrenceCursor = None
    userWorkingContextInfo: UserContextInfo = None
    filter: OccurrenceFilterOutput = None
    elementToPCIMap: ElementToProductContextInfoMap3 = None


@dataclass
class SelectedNewElementInfo(TcBaseObj):
    """
    This structure contains the list of new Awb0Element objects added. List of paged child occurrences and the cursor
    representing the point to which results have been retrieved.
    
    :var newElements: List of new Awb0Element created for the selected parent.
    :var pagedOccurrencesInfo: List of paged child occurrences and the cursor representing the point to which results
    have been retrieved.
    """
    newElements: List[Awb0Element] = ()
    pagedOccurrencesInfo: PagedOccurrencesInfo6 = None


@dataclass
class SourceContextInfo(TcBaseObj):
    """
    Configuration information e.g. Revision Rule name, Variant Rule name, Effectivity formula for the source context
    object.
    
    :var revRuleName: Display name of Revision Rule associated with the source context object
    :var varRuleName: Display name(s) of Variant Rule associated with the source context object.
    :var effectivityFormula: Display value of effectivity formula associated with the source context object
    """
    revRuleName: str = ''
    varRuleName: List[str] = ()
    effectivityFormula: str = ''


@dataclass
class AddObjectResp(TcBaseObj):
    """
    The structure contains parent objects under which object was added. When serviceData containing Awb0Element as
    created objects.
    
    :var selectedNewElementInfo: List of new Awb0Element objects and paged occurrences for input parent under which add
    was performed.
    :var newElementInfos: List of NewElementInfo2 when the parent is instantiated more than once.
    :var reloadContent: if true, the content must be reloaded to see updated information.
    :var serviceData: The service data containing created Awb0Element in the created list and partial error if any.
    """
    selectedNewElementInfo: SelectedNewElementInfo = None
    newElementInfos: List[NewElementInfo2] = ()
    reloadContent: bool = False
    serviceData: ServiceData = None


@dataclass
class EffectivityRange(TcBaseObj):
    """
    Structure to hold a row of effectivity data. This structure can also hold the intent values associated with the
    effecticity row data.
    
    :var unitIn: Unit at which this validity range starts. Valid value is non zero positive integer number.
    :var unitOut: Unit at which this validity range ends. Valid value is non zero positive integer number. Maximum
    value can be 2147483646 ( SO ) or 2147483647 ( UP ).
    :var dateIn: Date at which this validity range starts.
    :var dateOut: Date at which this validity range ends. Maximum value can be 26th Dec 9999 00:00:00 ( SO ) or 30th
    Dec 9999 00:00:00 ( UP ) in UTC timezone.
    :var intentFormula: The intent formula string associated with the current effectivivity row. This string represents
    all the intent values applied to the current effectivity range. This string should have following syntax:
    [Intent_item_ID]<Intent_family_name> = <Applied_family_value>
    Here,
    Intent_item_ID represents the item ID for the intent item associated with the Product Design.
    Intent_family_name, represents the name of the intent family selected, and
    Applied_family_value, is the selected value from intent family.
    For example, [IntentItem01]Manufacturing=Production.
    """
    unitIn: int = 0
    unitOut: int = 0
    dateIn: datetime = None
    dateOut: datetime = None
    intentFormula: str = ''


@dataclass
class NewElementInfo2(TcBaseObj):
    """
    This structure contains parent Awb0Element  under which new Awb0Element objects was created.
    
    :var parentElement: Parent under which objects was added.
    :var newElements: List of new Awb0Element created for the parent.
    :var newElementToPositionMap: Position of newly created Awb0Element within its parent.
    """
    parentElement: Awb0Element = None
    newElements: List[Awb0Element] = ()
    newElementToPositionMap: NewElementToPositionMap = None


@dataclass
class OccurrenceConfigInput(TcBaseObj):
    """
    OccurrenceConfigInput specifies the complete set of configuration rules including RevisionRule, Effectivity
    with/without intent values, and VariantRule to configure content. When Awb0ProductContextInfo is specified all
    other configuration parameters are ignored.
    
    :var productContext: Awb0ProductContextInfo containing configuration information.
    :var revisionRule: RevisionRule to configure the product.
    :var changeContext: Runtime Business object fnd0ChangeContext that holds the Change Context to be set and
    corresponding Change Configuration Mode.
    :var effectivityDate: Specifies the date effectivity to configure the product.
    :var unitNo: Unit number to use in order to configure the product.
    :var endItem: Specifies the end Item which governs the unit effectivity.
    :var variantRules: List of VariantRule(s) or StoredOptionSet(s) to configure the product.
    :var svrOwningProduct: This indicates the product(ItemRevision or CollaborativeDesign) associated with saved
    VariantRule.
    :var effectivityRanges: List of effectivity data. This data may or may not have effectivity intent values
    associated with the effectivity.
    :var occurrenceScheme: Occurrence scheme associated with product e.g Cpd0CollaborativeDesign.
    :var sourceContext: Source Context (Fnd0RecipeContainer or Mdl0BaselineRevision) associated with another
    Fnd0RecipeContainer
    """
    productContext: Awb0ProductContextInfo = None
    revisionRule: RevisionRule = None
    changeContext: RuntimeBusinessObject = None
    effectivityDate: datetime = None
    unitNo: int = 0
    endItem: BusinessObject = None
    variantRules: List[WorkspaceObject] = ()
    svrOwningProduct: BusinessObject = None
    effectivityRanges: List[EffectivityRange] = ()
    occurrenceScheme: BusinessObject = None
    sourceContext: WorkspaceObject = None


"""
The map which can have a key and value pair, used for occurrences expansion filters or criteria or options. The key and value are case sensitive. The allowed preference names are includeInterfaces. The allowed values are True/False.
"""
RequestPreference8 = Dict[str, List[str]]


"""
Map(WorkspaceObject, SourceContextInfo) containing pair of Source Context Object to its corresponding configuration information.
"""
SourceContextToInfoMap = Dict[WorkspaceObject, SourceContextInfo]


"""
Map containing Awb0Element to its respective Awb0ProductContextInfo object.
"""
ElementToProductContextInfoMap3 = Dict[Awb0Element, Awb0ProductContextInfo]


"""
Map containing newly created Awb0Element to its position within parent.
"""
NewElementToPositionMap = Dict[Awb0Element, int]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, BOMEdit, PSBOMViewRevision, ChangeItemRevision
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetNoteVariantOutput(TcBaseObj):
    """
    'GetNoteVariantOutput' structure contains object references to 1) the BOMEdit, 2) the associated change revision,
    3) the solution bvr, and 4) the impacted bvr.  It also contains a list of details count and a list of strings
    representing some textual details of the note or variant change.
    
    :var bomChange: An object reference to a BOMEdit
    :var changeRev: An object reference to a change revision.
    :var solutionBVR: An object reference to the Solution bvr.
    :var impactedBVR: An object reference to the Impacted bvr.
    :var vCount: Count of note or variant change details
    :var vBomChangeDetails: Note or variant change details.
    """
    bomChange: BOMEdit = None
    changeRev: ChangeItemRevision = None
    solutionBVR: PSBOMViewRevision = None
    impactedBVR: PSBOMViewRevision = None
    vCount: List[int] = ()
    vBomChangeDetails: List[str] = ()


@dataclass
class GetNoteVariantResponse(TcBaseObj):
    """
    'GetNoteVariantResponse' structure contains a list of 'GetNoteVariantOutput' structures and a standard service data.
    
    :var output: A reference to the list of GetNoteVariantOutput
    :var serviceData: Standard Service data.
    """
    output: List[GetNoteVariantOutput] = ()
    serviceData: ServiceData = None


@dataclass
class GetNoteVariantsInput(TcBaseObj):
    """
    GetNoteVariantsInput structure contains an object reference to a BOMEdit whose integer type is EITHER 6 (=Note
    Change) OR 7(=Variant Change) and a matching context string.
    
    :var bomChangeNode: An object reference to a BOMEdit
    :var contextRelName: A context string of two possible values: CM_note_change_details or CM_variant_change_details.
    """
    bomChangeNode: BusinessObject = None
    contextRelName: str = ''



from __future__ import annotations

from tcsoa.gen.Internal.AWS2._2019_06.Finder import FilterCriteriaInput
from typing import Dict, List
from tcsoa.gen.Internal.AWS2._2016_03.Finder import SearchSortCriteria
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from tcsoa.gen.Internal.AWS2._2017_12.Finder import SearchFilter4


@dataclass
class PropertyInfo(TcBaseObj):
    """
    A structure containing the internal and display names of a property.
    
    :var internalName: The internal name of the property.
    :var displayName: The display name of the property.
    """
    internalName: str = ''
    displayName: str = ''


@dataclass
class SearchInput4(TcBaseObj):
    """
    A structure containing input search criteria.
    
    :var providerName: The name of the search provider.
    :var searchCriteria: A map (string, string) used to define the search criteria to be performed by the provider.
    This is the same as the searchCriteria map in SearchInput structure used in Awp0::Soa::Internal::AWS2::_2016_03::
    Finder::performSearch operation.
    :var searchFilterMap: A map (string, list of SearchFilter4) containing the list of search filters for each search
    filter field. The key in the map is the property name that represents the filter category. It is in the format
    "TypeName.PropertyName". For example: WorkspaceObject.release_statuses. The value in the map is the value of the
    search filter.
    :var columnFilters: A list of filters (FilterCriteriaInput) to be applied on the columns.
    :var searchSortCriteria: The criteria to use to sort the results.
    """
    providerName: str = ''
    searchCriteria: StringMap4 = None
    searchFilterMap: SearchFilterMap7 = None
    columnFilters: List[FilterCriteriaInput] = ()
    searchSortCriteria: List[SearchSortCriteria] = ()


@dataclass
class ExportObjectsToFileResponse(TcBaseObj):
    """
    This structure contains the file ticket of exported file.
    
    :var serviceData: Partial errors are returned in the Service Data.
    :var fileTickets: A list of file tickets for the exported files.
    """
    serviceData: ServiceData = None
    fileTickets: List[str] = ()


@dataclass
class FilterFacetInput(TcBaseObj):
    """
    A structure containing input facet search criteria.
    
    :var columnName: The column name for which we need to obtain possible filter values (facets).
    :var providerName: The name of the data provider. This is the RuntimeBusinessObject type name.
    :var startIndex: The start index to return the facet values search results.
    :var maxToReturn: The maximum number of unique facet values to return.
    :var searchCriteria: A map (string, string) used to define the search criteria to be performed by the provider.
    This is the same as the searchCriteria map in SearchInput structure used in performSearch operation.
    :var columnFilters: A list of filters (FilterCriteriaInput) to be applied on the columns.
    :var searchFilterMap: A map (string, list of SearchFilter4) containing the list of search filters for each search
    filter field. The key in the map is the property name that represents the filter category. It is in the format
    "TypeName.PropertyName". For example: WorkspaceObject.release_statuses, the value in the map is the value of the
    search filter.
    """
    columnName: str = ''
    providerName: str = ''
    startIndex: int = 0
    maxToReturn: int = 0
    searchCriteria: StringMap4 = None
    columnFilters: List[FilterCriteriaInput] = ()
    searchFilterMap: SearchFilterMap7 = None


@dataclass
class FilterFacetResponse(TcBaseObj):
    """
    Structure containing the filter facet values and partial errors, if any.
    
    :var facetValues: A JSON string of the sorted list of unique facet values based on the query results from the data
    provider and the total unique facet values found in the search results. 
    The JSON will be in the following format, where "values" represents the sorted facet values for the given column
    and "totalFound" represents the total number of facet values found:
    {
        "values": [
            "facetValue1",
            "facetValue2",
            &hellip;
            "facetValueN"
        ],
        "totalFound": N
    }
    :var serviceData: Partial errors are returned in the Service Data.
    """
    facetValues: str = ''
    serviceData: ServiceData = None


"""
A map containing a list of search filters for each search filter field.
"""
SearchFilterMap7 = Dict[str, List[SearchFilter4]]


"""
Map from string to string.
"""
StringMap4 = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.Core._2008_03.Session import FavoritesInfo, FavoritesResponse, ConnectResponse
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class SessionService(TcService):

    @classmethod
    def connect(cls, featureKey: str, action: str) -> ConnectResponse:
        """
        Performs Teamcenter Flexlm license related operations, depending on the input parameters.
        
        The low level actions are:
        
        1.  ILM__init_module: Initializes the license module (if it has not already been initialized).
        2.  ILM__leave_module: Deallocates a license of the given module. If the user had N free licenses for this
        module, (N plus one) will be left after this call.
        3.  ILM__check_module: Checks to see if the user has bought the specified module and returns the number of
        purchased licenses.
        4.  ILM__enter_module: Allocates one license of the given module. If the user has bought N licenses for this
        module, (N minus one) will be left after this call.
        5.  ILM__exit_module: Leaves the module.
        """
        return cls.execute_soa_method(
            method_name='connect',
            library='Core',
            service_date='2008_03',
            service_name='Session',
            params={'featureKey': featureKey, 'action': action},
            response_cls=ConnectResponse,
        )

    @classmethod
    def setFavorites(cls, input: FavoritesInfo) -> ServiceData:
        """
        This operation saves new favorite containers and favorite objects for the current session user.
        
        Any partial errors encountered during this operation are returned using the 'clientID' specified by the caller.
        A service exception is thrown if an error is encountered that is not related to a specific favorite container
        or favorite object.  You can use favorites to track containers and objects you access frequently, such as
        folders, parts or forms.  For example, the Newstuff folder could be added as a container to the list of
        favorites.
        
        
        Use cases:
        User saves a container to their favorites list.
        
        For this operation, the list of all current favorites for the user and the list containing the container the
        user desires to add are supplied as input and the new container is added to the list of saved favorites in
        Teamcenter.
        
        
        Exceptions:
        >'Service Exception'    Thrown if an empty or invalid container or object input parameter is specified or the
        list of current favorites specified is no longer current.
        """
        return cls.execute_soa_method(
            method_name='setFavorites',
            library='Core',
            service_date='2008_03',
            service_name='Session',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def getFavorites(cls) -> FavoritesResponse:
        """
        This operation retrieves the saved Favorites containers and Favorites objects for the current session user. 
        You can use Favorites to track containers and objects you access frequently, such as folders, parts or forms.
        
        If errors are encountered, partial results are returned. Partial errors are returned with client IDs reflecting
        the index value of the saved Favorite. A service exception is thrown if an error is encountered that is not
        related to a specific Favorite container or Favorite object.
        
        Any Teamcenter object that is returned as a Favorite object is added to 'ServiceData' plain objects.  For
        example, if an item exists in the list of Favorite objects, the object tag value for that item will be returned
        in the 'ServiceData' list of plain objects.
        
        
        Use cases:
        User logs in and selects their saved Favorites.
        
        The Favorites view in the client application is populated with the Favorites containers and objects returned
        from this operation.
        
        
        Exceptions:
        >'Service Exception'    Thrown if the retrieval of saved favorites fails for the current user, the favorites
        preference value is missing a field or if the favorites object for the specified ID is not found.
        """
        return cls.execute_soa_method(
            method_name='getFavorites',
            library='Core',
            service_date='2008_03',
            service_name='Session',
            params={},
            response_cls=FavoritesResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.DebugMonitor._2015_07.DebugLogging import PerformanceJournalLevelResponse
from tcsoa.utils import TcService


class DebugLoggingService(TcService):

    @classmethod
    def setPerformanceJournalLevel(cls, level: int) -> bool:
        """
        Set the level of performance journaling
        
        Exceptions:
        >214610 The performance journaling level is invalid. Valid values are 0, 1, and 2.
        214611 The performance journaling level cannot be changed after journal entries have been written.
        """
        return cls.execute_soa_method(
            method_name='setPerformanceJournalLevel',
            library='Internal-DebugMonitor',
            service_date='2015_07',
            service_name='DebugLogging',
            params={'level': level},
            response_cls=bool,
        )

    @classmethod
    def getPerformanceJournalLevel(cls) -> PerformanceJournalLevelResponse:
        """
        Get the current level of performance journaling
        """
        return cls.execute_soa_method(
            method_name='getPerformanceJournalLevel',
            library='Internal-DebugMonitor',
            service_date='2015_07',
            service_name='DebugLogging',
            params={},
            response_cls=PerformanceJournalLevelResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExportObjectsResponse(TcBaseObj):
    """
    The ExportObjectsResponse is used as a response for performing the 'exportObjects' operation.
    
    :var sitesResponse: A list of ExportObjectsSiteResponseInfo
    :var serviceData: serviceData
    """
    sitesResponse: List[ExportObjectsSiteResponseInfo] = ()
    serviceData: ServiceData = None


@dataclass
class ExportObjectsSiteResponseInfo(TcBaseObj):
    """
    It is used for performing 'exportObjects' operation.
    
    :var siteId: Destination site id
    :var fileFmsTickets: A list of FMS ticket of output XML files.When there are a large number of exported objects,
    the export is broken up into multiple XML files
    :var logFmsTickets: A list of FMS ticket of output log files
    """
    siteId: int = 0
    fileFmsTickets: List[str] = ()
    logFmsTickets: List[str] = ()


@dataclass
class ImportObjectsResponse(TcBaseObj):
    """
    It is used as a response for performing 'importObjects' operation.
    
    :var fmsTicketOfFailedObjs: FMS ticket of file that contains a list of objects failed on import
    :var logFmsTicket: FMS ticket of log file
    :var servicedata: servicedata
    """
    fmsTicketOfFailedObjs: str = ''
    logFmsTicket: str = ''
    servicedata: ServiceData = None


@dataclass
class NamesAndValues(TcBaseObj):
    """
    The NamesAndValues structure represents a generic name-value pair.
    
    :var name: Name of the pair
    :var value: Value of the pair
    """
    name: str = ''
    value: str = ''


@dataclass
class OwningSiteAndObjs(TcBaseObj):
    """
    Vector of object and it's owning site.
    These objects need to be remote imported
    
    :var owningSiteId: owningSiteId
    :var objs: objs
    """
    owningSiteId: int = 0
    objs: List[BusinessObject] = ()


@dataclass
class StubReplicationInfo(TcBaseObj):
    """
    A stub is an object which acts as a place holder for actual object.This is used in multisite during data
    transfer,as sometimes we do not need all the dependent objects but it may not be possible to create the structure
    at target site without them.A stub serves the purpose in such situations.The StubReplicationInfo structure holds
    the stub objects that were replicated at masterSiteId site. The replicas are represented by by Global Service
    Identitiies  ( GSIdentites )in XML strings.
    
    
    :var masterSiteId: Master site id
    :var listOfGSIdentities: A list of objects were replicated
    """
    masterSiteId: int = 0
    listOfGSIdentities: List[str] = ()


@dataclass
class TransferFormula(TcBaseObj):
    """
    The TransferFormula structure contains the transfer options for performing an export.
    The transfer options can be TransferOptionSet, TransfferMode, list of options
    that override the ones specified in TransferOptionSet, and/or list of session options.
    Currently, the supported session options are 'OwnershipTransfer' and 'ContinueOnError'.
    
    :var optionSetName: Name of transfer option set.If option set is not specified TIEUnconfiguredExportDefault is used
    as default.
    :var optionSetUid: UID of transfer option set.If option set is not specified TIEExportDefaultTM transfer mode is
    used to export.
    :var optionOverrides: A list of override options.If the user wishes to change any default behaviour then these
    options are used to override.Example of a valid override option is
    - opt_entire_bom  False
    
    
    If the top item of an assembly is given as input, if the user does not want entire BOM to be processed then this is
    set to False.By default this value is set to True.
    :var sessionOptions: A list of session options.Some valid session options
    - ownership_transfer         yes or no
    - modified_objects_only     yes or no
    - revrule                         Latest Working or Any Status; Working or any rev rule name
    - svrule                         VariantRule Name
    - fastream                     yes
    - forceRetraverse             yes
    - ContinueOnError             yes
    
    
    :var transferModeName: Name of transfer mode.If not specified TIEExportDefaultTM will be taken as default.
    :var transferModeUid: UID of transfer mode.Name of transfer mode.No impact if this is not specified
    :var xsltFMSTicket: FMS ticket for the xsl file.No impact if this is not specified.
    :var reason: Reason for export.This is optional.
    """
    optionSetName: str = ''
    optionSetUid: str = ''
    optionOverrides: List[NamesAndValues] = ()
    sessionOptions: List[NamesAndValues] = ()
    transferModeName: str = ''
    transferModeUid: str = ''
    xsltFMSTicket: str = ''
    reason: str = ''


@dataclass
class CallToRemoteSiteResponse(TcBaseObj):
    """
    Remote exports objcets to specified sites. E-mail notification would be sent to
    current user if session option for email notification is set to true.
    
    :var msgIDs: msgIDs
    :var serviceData: serviceData
    """
    msgIDs: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class ChangeOwnershipInfo(TcBaseObj):
    """
    The ChangeOwnershipInfo has the list of objects whose ownership was transferred to replicaSiteId site.The
    transferred objects are represented by GSIdentites in XML strings.
    
    
    :var replicaSiteId: Replicated site id.Target site.
    :var listOfGSIdentities: A list of objects were transferred to replicaSiteId
    """
    replicaSiteId: int = 0
    listOfGSIdentities: List[str] = ()


@dataclass
class ConfirmExportResponse(TcBaseObj):
    """
    It is used as a response for performing confirmExport.
    
    :var ownershipChanges: A list of ChangeOwnershipInfo
    :var stubReplication: A list of stubReplication
    :var serviceData: serviceData
    """
    ownershipChanges: List[ChangeOwnershipInfo] = ()
    stubReplication: List[StubReplicationInfo] = ()
    serviceData: ServiceData = None


@dataclass
class DryRunExportResponse(TcBaseObj):
    """
    The dryRunExportResponse structure holds a string storing the File Management System(FMS) ticket of the report file
    which has information about the data that will be exported if real export was done.Any failure will be returned as
    partial errors.It is used as a response for performing 'dryRunExport'.
    
    :var fmsTicketForReport: FMS ticket of report file
    :var serviceData: serviceData
    """
    fmsTicketForReport: str = ''
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Internal.Visualization._2011_12.StructureManagement import ExpandPSFromOccurrenceListResponse, ExpandPSFromOccurrenceListPref, ExpandPSFromOccurrenceListInfo
from typing import List
from tcsoa.utils import TcService


class StructureManagementService(TcService):

    @classmethod
    def expandPSFromOccurrenceList(cls, info: List[ExpandPSFromOccurrenceListInfo], pref: ExpandPSFromOccurrenceListPref) -> ExpandPSFromOccurrenceListResponse:
        """
        This operation returns BOMLine(s) for the occurrences recorded in the occurrence list of the input occurrence
        object. Optionally, it can also return the objects of given type and relation that are attached to the objects
        that the BOMLine(s) represent (objectofBOMLine). 
        
        The operation can expand datasets and other objects related to parent and child BOMLine(s). Expansion of the
        related objects can be controlled by specifying a filter. The filter criteria supported are: relation name,
        related object type, and named references. 
        
        This operation allows for expansion to reference object associated to a named reference. Typically this is a
        file and in that case a FMS ticket will be returned to provide access to this file. Where a named reference
        points to a file, this operation allows caller to specify from a defined set of handler options, which specific
        handler should be used in choosing file(s) to return. This is specified through the input parameter
        'NamedRefHandler' (included in the info object). The service would also return the side car jt referenced via
        the pmi0blpmi_bl_jt_tags property on the BOMLine. This information is returned only if the input to the
        'NamedRefHandler' is mentioned as PreferredJt and if the pmi template is deployed in the database
        
        Use cases:
        When the user wants to expand a specific list of occurrences into an existing BOMWindow that contains the
        parent BOMLine of the occurrences.
        This service is used to support the following primary use cases.
        
        Pruned launch of selected lines to visualization
        - User opens structure in Structure Manager/Multi Structure Manager/Manufacturing Process Planner, configures
        it, selects some lines, and sends those lines to Lifecycle Viewer or integrated standalone visualization
        - The launching client (system) calls createVisSCsFromBOMs to record the selected lines and BOM configuration
        information
        - The visualization client receives the request to open the selected lines as an object reference to a
        VisStructureContext object
        - The visualization client calls the createBOMsFromRecipes operation and passes the VisStructureContext object
        reference
        - The system creates a BOMWindow and configures it properly (to match launching configuration)
        - The client pulls the occurrence_list from the VisStructureContext object
        - The client issues an expandPSFromOccurrenceList call to load the occurrences
        - The client(system) loads the selected occurrences in the visualization client, the structure is pruned to
        contain only those occurrences sent
        
        
        
        Product View launch to visualization
        - 1.    User selects a Product View in the Product View Gallery of embedded visualization inside Structure
        Manager/Multi Structure Manager/Manufacturing Process Planner, or in My Teamcenter and sends it to integrated
        standalone TcVis or Lifecycle Viewer
        - 2.    The visualization client receives the request to open the selected product View
        - 3.     The visualization client interrogates the Product View data model, fetches the files from the dataset,
        and gets a list of visible lines for the Product View
        - 4.    The client issues an expandPSFromOccurrenceList call for all visible lines referenced by the Product
        View
        - 5.    The client loads the visible lines then applies the product view
        
        
        
        Use Case Dependencies: 
        The expandPSFormOccurrenceList operation is called for existing parent BOMLines within a BOMWindow (i.e. the
        parent BOMLine(s) must already exist within a created BOMWindow) and also requires a previously
        captured/defined list of occurrences represented as a list of UID strings.
        
        The following services are used in conjunction with expandPSFromOccurrenceList to complete the use cases above.
        Teamcenter::Soa::Internal::Visualization::_2008_06::StructureManagement. createVisSCsFromBOMs
        Teamcenter::Soa::Internal::Visualization::_2008_06::StructureManagement. createBOMsFromRecipes
        Teamcenter::Soa::Internal::Visualization::_2008_06::StructureManagement. areRecipesMergible
        Teamcenter::Soa::Internal::Visualization::_2010_09::DataManagement. getSnapshot3DInfo
        """
        return cls.execute_soa_method(
            method_name='expandPSFromOccurrenceList',
            library='Internal-Visualization',
            service_date='2011_12',
            service_name='StructureManagement',
            params={'info': info, 'pref': pref},
            response_cls=ExpandPSFromOccurrenceListResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Role, User, BusinessObject, TC_Project, Group
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class UserGroupRoleInfo(TcBaseObj):
    """
    The UserGroupRoleInfo structure contains the user, the group, and the role. The group and the role are optional.
    
    :var tcUser: The User object.
    :var tcGroup: The Group object.
    :var tcRole: The Role object.
    """
    tcUser: User = None
    tcGroup: Group = None
    tcRole: Role = None


@dataclass
class UserProjects(TcBaseObj):
    """
    The structure which holds the TC_Project objects for the associated UserGroupRoleInfo structure
    
    :var userGroupRole: The UserGroupRoleInfo structure.
    :var projects: The TC_Project objects that's associated with the UserGroupRoleInfo.
    """
    userGroupRole: UserGroupRoleInfo = None
    projects: List[TC_Project] = ()


@dataclass
class UserProjectsResponse(TcBaseObj):
    """
    The structure which holds the ServiceData and all the TC_Project objects that are associated with the given
    UserGroupRoleInfo
    
    :var userProjectList: A list of UserProjects structure.
    :var serviceData: The ServiceData.
    """
    userProjectList: List[UserProjects] = ()
    serviceData: ServiceData = None


@dataclass
class ChangeOwningProgramInput2(TcBaseObj):
    """
    Contains data used for changing the owning program for the  given set of objects.
    
    :var owningProgram: The project (TC_Project) to set as the Owning Program of the target objects.
    :var inputObjects: A list of WorkspaceObject for which the Owning Program is to be changed.
    """
    owningProgram: TC_Project = None
    inputObjects: List[BusinessObject] = ()
